#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "polylib/types.h"
#include "polylib/matrix.h"
#include "polylib/polyhedron.h"

int dim,
size;

/*
Fonction : read_hedr
Rle :	lit sur l'entre un arbre dcrivant un semi-polydre.
Out  :	Polyhedron * = le semi-polydre calcul par polylib.
Note :	le format de l'entre standard doit tre le suivant :
	S ::= ("L\n" int^size) | (["U"|"I"] (int n) "\n" S^n)
	par exemple :
	  I 2
	  U 1
	  L
	  1 2 3 4 5
	  L
	  1 2 3 4 5
*/
Polyhedron *read_hedr() {
  Polyhedron *p = NULL;
  char c;
  int i;
  int nb = 0;
  char s[256];

  if (fgets(s, 256, stdin) == NULL) abort();
  i = sscanf(s, "%c %d", &c, &nb);
  if (c == 'L' && i == 1) {
    Matrix *m = Matrix_Alloc(1, size + 2);
    Matrix_Read_Input(m);
    p = Constraints2Polyhedron(m, 600);
    Matrix_Free(m);
    return p;
  }
  if ((c != 'U' && c != 'I') || i != 2) abort();
  for(i = 0; i < nb; i++) {
    Polyhedron *t1 = read_hedr();
    if (p == NULL) p = t1;
    else {
      Polyhedron *t2;
      t2 = (c == 'U') ? DomainUnion(p, t1, 600) : DomainIntersection(p, t1, 600);
      Domain_Free(p);
      Domain_Free(t1);
      p = t2;
    }
  }
  if (p == NULL) abort();
  //p = (c == 'U') ? Empty_Polyhedron(size) : Universe_Polyhedron(size);

  return p;
}

/*
Fonction : print_indet
Rle :	affiche une indtermine tant donn son numro.
In   :	int = le numro de l'indtermine.
*/
void print_indet(int n) {
  int i = n / dim;
  int j = n % dim;
  if (i < dim) printf("a%d%d", i, j);
  else if (i < dim + 1) printf("b%i", j);
  else printf("c%i", j);
}

/*
Fonction : print_aff
Rle :	affiche une fonction affine en les indtermines.
In   :	Value[size + 1] = (size + 1) entiers.
Note :	le dernier des entiers correspond au coefficient constant de la fonction.
*/
void print_aff(int *c) {
  int i, a;
  int b = 0;

  for(i = 0; i < size; i++) {
    a = c[i];
    if (a == 0) continue;
    if (b) {
      if (a < 0) { printf(" - "); a = -a; }
      else printf(" + ");
    } else if (a == -1) {
      printf("- ");
      a = -a;
    }
    if (a != 1) printf("%d.", c[i]);
    print_indet(i);
    b = 1;
  }
  a = c[i];
  if (a != 0) {
    if (b) {
      if (a < 0) { printf(" - "); a = -a; }
      else printf(" + ");
    }
    printf("%d", a);
  }
}

/*
Fonction : main
Rle :	calcule et affiche les systmes d'quations traduisant les arbres
	fournis sur l'entre standard.
Note :	le format de l'entre standard doit tre le suivant :
	E ::= (int dim) "\n" (string inf) "\n" T^* "E\n"
	T ::= "T" ["I"|"."]^size "\n" S
	par exemple :
	  1
	  -inf
	  T .II
	  I 2
	  U 1
	  L
	  1 2 3 4 5
	  L
	  1 2 3 4 5
	  E
*/
int main() {
  Polyhedron *p;
  char s[256];
  char inf[256];

  if (fgets(s, 256, stdin) == NULL) abort();
  if (sscanf(s, "%d", &dim) != 1) abort();
  if (fgets(inf, 256, stdin) == NULL) abort();
  size = dim * (2 + dim);

  puts("----------");
  for(;;) {
    if (fgets(s, 256, stdin) == NULL) abort();
    if (s[0] == 'E') break;
    if (strlen(s) < size + 2 || s[0] != 'T') abort();
    p = read_hedr();
    for(; p != NULL; p = p->next) {
      int j;
      if (emptyQ(p)) continue;
      for(j = 0; j < size; j++)
	if (s[2 + j] == 'I') {
	  print_indet(j);
	  printf(" = %s", inf);
	}
      for(j = 0; j < p->NbConstraints; j++) {
	print_aff(p->Constraint[j] + 1);
	if (p->Constraint[j][0] == 0) puts(" = 0"); else puts(" >= 0");
      }
      puts("----------");
    }
    Domain_Free(p);
  }
  return 0;
}
