/* matrix.c 
     COPYRIGHT
          Both this software and its documentation are

              Copyright 1993 by IRISA /Universite de Rennes I - France,
              Copyright 1995,1996 by BYU, Provo, Utah
                         all rights reserved.

          Permission is granted to copy, use, and distribute
          for any commercial or noncommercial purpose under the terms
          of the GNU General Public license, version 2, June 1991
          (see file : LICENSING).
*/

#include <stdio.h>
#include <ctype.h>
#include "types.h"
#include "vector.h"
#include "matrix.h"

void errormsg1(char *f , char *msgname, char *msg);

int Pol_status;

/* #define DEBUG 1 */

#define Vector_Copy(p1, p2, length) \
  memcpy((char *)(p2), (char *)(p1), (int)(length)*sizeof(int))

#define Vector_Init(p1, length) \
  memset((char *)(p1), 0, (int)(length)*sizeof(int))

#define Vector_Exchange(a, b, t, l)\
{\
  memcpy((char *)(t), (char *)(a), (l)*sizeof(int));\
  memcpy((char *)(a), (char *)(b), (l)*sizeof(int));\
  memcpy((char *)(b), (char *)(t), (l)*sizeof(int));\
}

Matrix *Matrix_Alloc(NbRows, NbColumns)
     unsigned NbRows, NbColumns;
{
  Matrix *Mat;
  int **q, *p, i;

  Mat=(Matrix *)malloc(sizeof(Matrix));
  if (!Mat) return 0;
  Mat->NbRows=NbRows;
  Mat->NbColumns=NbColumns;
  Mat->p=q=(int **)malloc(NbRows * sizeof(int *));
  if (!q)
  {  free(Mat);
     return 0;
  }
  Mat->p_Init=p=(int *)malloc(NbRows * NbColumns * sizeof(int));
  if (!p)
  {  free(Mat);
     free(q);
     return 0;
  }
  for (i=0;i<NbRows;i++) {
    *q++=p;
    p+=NbColumns;
  }
  return Mat;
} /* Matrix_Alloc */


void Matrix_Free(Mat)
     Matrix *Mat;
{ if (!Mat) return;
  free((char *) Mat->p_Init);
  free((char *) Mat->p);
  free((char *) Mat);
} /* Matrix_Free */


void Matrix_Vector_Product(mat, p1, p2)
     Matrix *mat;
     int *p1, *p2;
{
  int **cm, *q, *cp1, *cp2, NbRows, NbColumns, i, j;

  NbRows=mat->NbRows;
  NbColumns=mat->NbColumns;
  
  cm=mat->p;
  cp2=p2;
  for (i=0;i<NbRows;i++) {
    q=*cm++;
    cp1=p1;
    *cp2=*q++ * *cp1++;
    for (j=1;j<NbColumns;j++)
      *cp2+= *q++ * *cp1++;
    cp2++;
  }
} /* Matrix_Vector_Product */

void Vector_Matrix_Product(p1, mat, p2)
     Matrix *mat;
     int *p1, *p2;
{
  int **cm, *cp1, *cp2, NbRows, NbColumns, i, j;
  
  NbRows=mat->NbRows;
  NbColumns=mat->NbColumns;
  cp2=p2;
  cm=mat->p;
  for (j=0;j<NbColumns;j++) {
    cp1=p1;
    *cp2=*(*cm+j) * *cp1++;
    for (i=1;i<NbRows;i++)
      *cp2+=*(*(cm+i)+j) * *cp1++;
    cp2++;
  }
} /* Vector_Matrix_Product */

void Matrix_Product(mat1, mat2, mat3)
     Matrix *mat1, *mat2, *mat3;
{
  int **q1, **q2, *p1, *p3, Size, sum, i, j, k;
  unsigned NbRows, NbColumns;

  NbRows    = mat1->NbRows;
  NbColumns = mat2->NbColumns;
  /* mat3 = Matrix_Alloc(NbRows, NbColumns); */
  Size      = mat1->NbColumns;
  if (mat2->NbRows!=Size || mat3->NbRows!=NbRows || mat3->NbColumns!=NbColumns)
  {  fprintf(stderr, "? Matrix_Product : incompatable matrix dimension\n");
     return;
  }
  
  p3=mat3->p_Init;
  q1=mat1->p;
  q2=mat2->p;
  for (i=0;i<NbRows;i++) {
    for (j=0;j<NbColumns;j++) {
      p1=*(q1+i);
      sum=0;
      for (k=0;k<Size;k++)
        sum+=*p1++ * *(*(q2+k)+j);
      *p3++=sum;
    }
  }
} /* Matrix_Product */

void Matrix_Print(Dst, Format, Mat)
     FILE* Dst;
     char *Format;
     Matrix *Mat;
{
  int *p, i, j;
  unsigned NbRows, NbColumns;

  fprintf(Dst,"%d %d\n", NbRows=Mat->NbRows, NbColumns=Mat->NbColumns);
  for (i=0;i<NbRows;i++) {
    p=*(Mat->p+i);
    for (j=0;j<NbColumns;j++)
      fprintf(Dst, Format, *p++);
      fprintf(Dst, "\n");
  }
} /* Matrix_Print */

void Matrix_Read_Input(Matrix *Mat) {
  
  int *p;
  int i,j,n;
  char *c, s[1024];
  
  p = Mat->p_Init;
  for (i=0;i<Mat->NbRows;i++) {
    do {
      c = fgets(s, 1024, stdin);
      while(isspace(*c) && *c!='\n')
	++c;
    } while(c && (*c =='#' || *c== '\n'));
    
    if (!c) {
      errormsg1( "Matrix_Read", "baddim", "not enough rows" );
      break;
    }
    for (j=0;j<Mat->NbColumns;j++) {
      if(!c || *c=='\n' || *c=='#') {
	errormsg1("Matrix_Read", "baddim", "not enough columns");
	break;
      }
      if (sscanf(c," %d%n",p++,&n) == 0) {
	errormsg1( "Matrix_Read", "baddim", "not enough columns" );
	break;
      }
      c += n;
    }
  }
} /* Matrix_Read_Input */

Matrix *Matrix_Read(void) {
  
  Matrix *Mat;
  unsigned NbRows, NbColumns;
  char s[1024];
  
  while(fgets(s, 1024, stdin)==0);
  while ((*s=='#' || *s=='\n') ||
	 (sscanf(s, "%d %d", &NbRows, &NbColumns)<2))
    fgets(s, 1024, stdin);
  Mat = Matrix_Alloc(NbRows,NbColumns);
  if(!Mat) {
    errormsg1("Matrix_Read", "outofmem", "out of memory space");
    return(NULL);
  }
  Matrix_Read_Input(Mat);
  return Mat;
} /* Matrix_Read */

/* basic hermite engine */
static int hermite(H, U, Q)
Matrix *H, *U, *Q;
{  int i, j, k, nc, nr, x, reduced;
   int rank, pivot, pivotrow;
   int *tmp, *tmp2;

   /*                     T                     -1   T */
   /* Computes form: A = Q H  and U A = H  and U  = Q  */

   if (!H)
   {  errormsg1("Domlib", "nullH", "hermite: ? Null H");
      return -1;
   }
   nc = H->NbColumns;
   nr = H->NbRows;
   tmp = (int *) malloc(nc * sizeof(int));
   tmp2 = (int *) malloc(nr * sizeof(int));
   if (!tmp ||!tmp2)
   {  errormsg1("Domlib", "outofmem", "out of memory space");
      return -1;
   }

#ifdef DEBUG
  fprintf(stderr,
  "Start  -----------\n");
  Matrix_Print(stderr,"%4d", H);
#endif

   for (k=0, rank=0; k<nc && rank<nr; k=k+1)
   {  reduced = 1;	/* go through loop the first time */
#ifdef DEBUG
  fprintf(stderr, "Working on col %d.  Rank=%d ----------\n", k+1, rank+1);
#endif
      while (reduced)
      {  reduced=0;

         /* 1. find pivot row */
         pivot = abs(H->p[rank][k]);  /* the kth-diagonal element */
         pivotrow = rank;
         /* find the row i>rank with smallest nonzero element in col k */
         for (i=rank+1; i<nr; i++)
         {  x = abs(H->p[i][k]);
            if ( x!=0 && (x<pivot || pivot==0) ) {pivot = x; pivotrow = i;}
         }

         /* 2. bring pivot to diagonal (exchange rows pivotrow and rank) */
         if (pivotrow != rank)
         {  Vector_Exchange(H->p[pivotrow], H->p[rank], tmp, nc );
            if (U)
              Vector_Exchange(U->p[pivotrow], U->p[rank], tmp2, nr );
            if (Q)
              Vector_Exchange(Q->p[pivotrow], Q->p[rank], tmp2, nr );
#ifdef DEBUG
  fprintf(stderr,
  "Exchange rows %d and %d  -----------\n", rank+1, pivotrow+1);
  Matrix_Print(stderr,"%4d", H);
#endif
         }
         pivot = H->p[rank][k];	/* actual ( no abs() ) pivot */

         /* 3. Invert the row 'rank' if pivot is negative */
         if (pivot < 0)
         {  pivot = -pivot;
            for (j=0; j<nc; j++) H->p[rank][j] = -(H->p[rank][j]);
            if (U)
              for (j=0; j<nr; j++) U->p[rank][j] = -(U->p[rank][j]);
            if (Q)
              for (j=0; j<nr; j++) Q->p[rank][j] = -(Q->p[rank][j]);
#ifdef DEBUG
  fprintf(stderr,
  "Negate row %d  -----------\n", rank+1);
  Matrix_Print(stderr,"%4d", H);
#endif
         }

         if (pivot != 0)
         {  /* 4. Reduce the column modulo the pivot */
            /*    This eventually zeros out everything below the */
            /*    diagonal and produces an upper triangular matrix */
            for (i=rank+1; i<nr; i++)
            {  x = H->p[i][k];
               if (x)
	       {  /* floor[integer division] (corrected for neg x) */
                  if ((x<0) && (x%pivot!=0)) x=(x/pivot)-1; else x/=pivot;
                  for (j=0; j<nc; j++) H->p[i][j] -= (x * H->p[rank][j]);
                  if (U)
                    for (j=0; j<nr; j++) U->p[i][j] -= (x * U->p[rank][j]);
                  if (Q)
                    for (j=0; j<nr; j++) Q->p[rank][j] += (x * Q->p[i][j]);
                  reduced = 1;
#ifdef DEBUG
  fprintf(stderr,
  "row %d = row %d - %d row %d -----------\n", i+1, i+1, x, rank+1);
  Matrix_Print(stderr,"%4d", H);
#endif
               } /* if (x) */
            } /* for (i) */
         } /* if (pivot != 0) */
      } /* while (reduced) */

      /* Last finish up this column */
      /* 5. Make pivot column positive (above pivot row) */
      /*    x should be zero for i>k */
      if (pivot!=0)
      {  for (i=0; i<rank; i++)
         {  x = H->p[i][k];
            if (x)
            {  /* floor[integer division] (corrected for neg x) */
               if ((x<0) && (x%pivot!=0)) x=(x/pivot)-1; else x/=pivot;
               for (j=0; j<nc; j++) H->p[i][j] -= x * H->p[rank][j];
               if (U)
                 for (j=0; j<nr; j++) U->p[i][j] -= x * U->p[rank][j];
               if (Q)
                 for (j=0; j<nr; j++) Q->p[rank][j] += x * Q->p[i][j];
#ifdef DEBUG
  fprintf(stderr,
  "row %d = row %d - %d row %d -----------\n", i+1, i+1, x, rank+1);
  Matrix_Print(stderr,"%4d", H);
#endif
            } /* if (x) */
         } /* for (i) */
         rank++;
      } /* if (pivot!=0) */
   } /* for (k) */
   free(tmp2);
   free(tmp);
   return rank;
}

void right_hermite(A, Hp, Up, Qp)
Matrix *A, **Up, **Qp, **Hp;
{  Matrix *H, *Q, *U;
   int i, j, nr, nc, tmp, rank;

   /* Computes form: A = QH , UA = H */

   nc = A->NbColumns;
   nr = A->NbRows;

   /* H = A */
   *Hp = H = Matrix_Alloc(nr, nc);
   if (!H)
   {  errormsg1("DomRightHermite", "outofmem", "out of memory space");
      return;
   }

   Vector_Copy(A->p_Init, H->p_Init, nr*nc );

   /* U = I */
   if (Up)
   {  *Up = U = Matrix_Alloc(nr, nr);
      if (!U)
      {  errormsg1("DomRightHermite", "outofmem", "out of memory space");
         return;
      }
      Vector_Init(U->p_Init, nr*nr);             /* zero's */
      for (i=0; i<nr; i++) U->p[i][i] = 1;       /* with diagonal of 1's */
   }
   else U=(Matrix *)0;

   /* Q = I */
   /* Actually I compute Q transpose... its easier */
   if (Qp)
   {  *Qp = Q = Matrix_Alloc(nr, nr);
      if (!Q)
      {  errormsg1("DomRightHermite", "outofmem", "out of memory space");
         return;
      }
      Vector_Init(Q->p_Init, nr*nr);            /* zero's */
      for (i=0; i<nr; i++) Q->p[i][i] = 1;      /* with diagonal of 1's */
   }
   else Q=(Matrix *)0;

   rank = hermite(H,U,Q);
   /* Q is returned transposed */

   /* Transpose Q */
   if (Q)
   {  for (i=0; i<nr; i++)
      {  for (j=i+1; j<nr; j++)
         {  tmp = Q->p[i][j];
            Q->p[i][j] = Q->p[j][i];
            Q->p[j][i] = tmp;
         }
      }
   }
}

void left_hermite(A, Hp, Up, Qp)
Matrix *A, **Hp, **Qp, **Up;
{  Matrix *H, *HT, *Q, *U;
   int i, j, nc, nr, tmp, rank;

   /* Computes left form: A = HQ , AU = H , 
                           T    T T    T T   T
        using right form  A  = Q H  , U A = H */

   nr = A->NbRows;
   nc = A->NbColumns;

   /* HT = A transpose */
   HT = Matrix_Alloc(nc, nr);
   if (!HT)
   {  errormsg1("DomLeftHermite", "outofmem", "out of memory space");
      return;
   }
   for (i=0; i<nr; i++)
      for (j=0; j<nc; j++)
         HT->p[j][i] = A->p[i][j];

   /* U = I */
   if (Up)
   {  *Up = U = Matrix_Alloc(nc, nc);
      if (!U)
      {  errormsg1("DomLeftHermite", "outofmem", "out of memory space");
         return;
      }
      Vector_Init(U->p_Init, nc*nc);            /* zero's */
      for (i=0; i<nc; i++) U->p[i][i] = 1;      /* with diagonal of 1's */
   }
   else U=(Matrix *)0;

   /* Q = I */
   if (Qp)
   {  *Qp = Q = Matrix_Alloc(nc, nc);
      if (!Q)
      {  errormsg1("DomLeftHermite", "outofmem", "out of memory space");
         return;
      }
      Vector_Init(Q->p_Init, nc*nc);            /* zero's */
      for (i=0; i<nc; i++) Q->p[i][i] = 1;      /* with diagonal of 1's */
   }
   else Q=(Matrix *)0;

   rank = hermite(HT, U, Q);

   /* H = HT transpose */
   *Hp = H = Matrix_Alloc(nr, nc);
   if (!H)
   {  errormsg1("DomLeftHermite", "outofmem", "out of memory space");
      return;
   }
   for (i=0; i<nr; i++)
      for (j=0; j<nc; j++)
         H->p[i][j] = HT->p[j][i];
   Matrix_Free(HT);

   /* Transpose U */
   if (U)
   {  for (i=0; i<nc; i++)
      {  for (j=i+1; j<nc; j++)
         {  tmp = U->p[i][j];
            U->p[i][j] = U->p[j][i];
            U->p[j][i] = tmp;
         }
      }
   }
}

/*----------------------------------------------------------------------*/
/* MatInverse                                                           */
/*      Matrix Inversion (integer -> rational).                         */
/*      Input : Matrix M  k*k >>WARNING: This matrix is modified        */
/*	Output : Matrix MInv  k*(k+1)  >>Must be preallocated           */
/*          The last column is the common denominator for each line     */
/*      Returns : 1 if ok, 0 if M not invertible.                       */
/*----------------------------------------------------------------------*/
int MatInverse( M, MInv )
Matrix *M, *MInv;
{
	int p, k, li, x, piv, c, g;
	int *tmp;

	if( M->NbRows != M->NbColumns )
	{
		fprintf( stderr, "Trying to invert a non-square matrix !\n");
		return 0;
	}
	k = M->NbRows;
	tmp = (int *) malloc( (k+1)*sizeof(int) );

	/* initialise MInv */
	memset( &MInv->p[0][0], 0, sizeof(int)*k*(k+1) );
	for( p=0 ; p<k ; ++p )
	{
		MInv->p[p][p] = 1;	/* diagonal */
		MInv->p[p][k] = 1;	/* denum */
	}

	/* gauss-jordan on the 2 matrices in parallel. */
	for( p=0 ; p<k ; ++p )
	{
		if (M->p[p][p]==0)	/* need a new pivot */
		{	for( li=p ; li<k ; ++li )	/* search for one */
				if( M->p[li][p] )
					break;
			if( li==k )		/* no pivot found... */
				return 0;

			/* exchange of the 2 lines (in the 2 matrices...)*/
			for( c=0 ; c<k ; ++c )
			{
				x = M->p[li][c];
				M->p[li][c] = M->p[p][c];
				M->p[p][c] = x;

				x = MInv->p[li][c];
				MInv->p[li][c] = MInv->p[p][c];
				MInv->p[p][c] = x;
			}
		}

		/* zero-out everything but the pivot */
		for( li=0 ; li<k ; ++li )
		{
			if (li==p) continue;	/* skip the pivot */
			if( (x=M->p[li][p]) )
			{
				piv = M->p[p][p];
				g = Gcd( x, piv );
				if (g!=1)
				{	x /= g;
					piv = piv/g;
				}
				for( c=(li>p ? p : 0) ; c<k ; ++c )
					M->p[li][c] = piv*M->p[li][c] - x*M->p[p][c];
				for( c=0 ; c<k ; ++c )
					MInv->p[li][c] = piv*MInv->p[li][c] - x*MInv->p[p][c];

				/* simplification if possible */
				g = Gcd( Vector_Gcd(&MInv->p[li][0],tmp,k),
					Vector_Gcd(&M->p[li][0],tmp,k) );
				if( g != 1 )
					for( c=0 ; c<k ; ++c )
					{
						M->p[li][c] /= g;
						MInv->p[li][c] /= g;
					}
			}
		}
	}

	/* ok... Now simplify each line (common denom) */
	for( li=0 ; li<k ; ++li )
	{
		MInv->p[li][k] = M->p[li][li];
		/* denom > 0 */
		Vector_Normalize_Positive( &MInv->p[li][0], tmp, k+1, k);
	}
	free(tmp);

	return 1;
}

/*----------------------------------------------------------------------*/
/* rat_prodmat                                                          */
/*      Matrix Product (integer X rational -> rational)                 */
/*      Outputs: S result (rational matrix) >> must be preallocated     */
/*      Inputs:  X (integer matrix)                                     */
/*               P (rational matrix)                                    */
/*----------------------------------------------------------------------*/
void rat_prodmat( S, X, P )
Matrix *S, *X, *P;
{
	int i,j,k;
	int denom = P->NbColumns - 1;
	int dcom, olddc, g, d, s, *tmp;

	tmp = (int *) malloc ( P->NbColumns * sizeof(int) );

	/* compute the lcm of the denominators */
	dcom = P->p[0][denom];	/* diviseur commun */
	for( k=1 ; k<P->NbRows ; ++k )
	{
		olddc = dcom;
		d = P->p[k][denom];
		g = Gcd( dcom, d );
		dcom *= (d/g);
		if( (dcom/d)*g != olddc )
		{
			fprintf( stderr, "Multiplication overflow in the computing of Si!\n" );
			dcom = 0;
			break;
		}
	}

	for( i=0 ; i<X->NbRows ; ++i )
		for( j=0 ; j<P->NbColumns-1 ; ++j )
		{
			s = 0;
			for( k=0 ; k<P->NbRows ; ++k )
				s += X->p[i][k] * P->p[k][j] *
				(dcom?(dcom / P->p[k][denom]):1);
			S->p[i][j] = s;
		}

	for( i=0 ; i<S->NbRows ; ++i )
	{
		S->p[i][denom] = dcom;
		/* normalize so that last element >=0 */
		Vector_Normalize_Positive( &S->p[i][0], tmp, S->NbColumns, S->NbColumns-1 );
	}
	free(tmp);
}

/*----------------------------------------------------------------------*/
/* Matrix_Inverse                                                       */
/*      Matrix Inversion (rational -> rational).                        */
/*      Input :  Matrix M    k*k     >>WARNING: This matrix is modified */
/*	Output : Matrix MInv k*k     >>Must be preallocated             */
/*          The last column of last row is the common denominator       */
/*      Returns : 1 if ok, 0 if M not invertible.                       */
/*----------------------------------------------------------------------*/
int Matrix_Inverse( M, MInv )
Matrix *M, *MInv;
{
   int p, k, li, x, piv, c, g;
   int *tmp, *den;

   if( M->NbRows != M->NbColumns )
   {  fprintf( stderr, "Trying to invert a non-square matrix !\n");
      return 0;
   }
   k = M->NbRows;
   tmp = (int *) malloc( (k)*sizeof(int) );	/* for row exchanges */

   /* initialise MInv to Identity */
   memset( &MInv->p[0][0], 0, sizeof(int)*k*k );
   for( p=0 ; p<k ; ++p ) MInv->p[p][p] = 1;	/* diagonal */

   /* gauss-jordan on the 2 matrices in parallel. */
   for( p=0 ; p<k ; ++p )
   {  if (M->p[p][p]==0)			/* need a new pivot */
      {  for( li=p ; li<k ; ++li )		/* search for one */
            if( M->p[li][p] ) break;
         if( li==k ) return 0;			/* no pivot found... */

         /* exchange of row p and row li (in both matrices...)*/
         for( c=0 ; c<k ; ++c )			/* for all k columns */
         {  x = M->p[li][c];
            M->p[li][c] = M->p[p][c];
            M->p[p][c] = x;

            x = MInv->p[li][c];
            MInv->p[li][c] = MInv->p[p][c];
            MInv->p[p][c] = x;
         }
      }

      /* zero-out everything but the pivot */
      for( li=0 ; li<k ; ++li )			/* for all k rows */
      {  if (li==p) continue;			/* skip the pivot */
         if( (x=M->p[li][p]) )			/* if non-zero, then zero out */
         {  piv = M->p[p][p];
            g = Gcd( x, piv );
            if (g!=1)				/* need to scale rows */
            {  x /= g;
               piv = piv/g;
            }
            for( c=(li>p ? p : 0) ; c<k ; ++c )
               M->p[li][c]    = piv*M->p[li][c] - x*M->p[p][c];
            for( c=0 ; c<k ; ++c )
               MInv->p[li][c] = piv*MInv->p[li][c] - x*MInv->p[p][c];

            /* simplify if possible */
            g = Gcd( Vector_Gcd(&MInv->p[li][0],tmp,k),
                     Vector_Gcd(   &M->p[li][0],tmp,k) );
            if( g != 1 )
            for( c=0 ; c<k ; ++c )
            {  M->p[li][c] /= g;
               MInv->p[li][c] /= g;
            }
         }
      }
   }

   /* Find common denom for each row */
   den = (int *)malloc(k*sizeof(int));
   x = 1;
   for( li=0 ; li<k ; ++li )
   {  den[li]=M->p[li][li];
      g = Vector_Gcd( &MInv->p[li][0], tmp, k);	/* g is always positive */
      g = Gcd(g, den[li]);
      if ( den[li]<0 ) g=-g;			/* make denominator positive */
      if ( g!=1 )
      {  for (c=0; c<k; c++) MInv->p[li][c]/=g;	/* normalize */
         den[li]/=g;
      }
      g=Gcd(x,den[li]);
      x*=(den[li]/g);
   }
   if (x!=1)
     for( li=0 ; li<k ; ++li )
     for (c=0; c<k; c++) MInv->p[li][c]*=(x/den[li]);    /* normalize */
   free(den);
   free(tmp);
   return 1;
}

